from ..analyzer.imports import *
from .thread import *
from .write import Write_file
from .swy import *

def TestAnalyzer(path):
	since = -1
	for method in ['0', '1']:
		log.Info('Method: ', method)
		for anal in analyzers:
			url_tests = anal.TEST
			for sub in url_tests:
				file = Write_file(None, None, 'html', method, since)
				url = {anal.SITE: [sub]}
				log.RInfo('Start sub="', sub, '", method=', method)
				Run_analyze(url, False, file, method)
				if len(file.contents) == 0:
					log.RWarning('{}: No content found'.format(sub))
				else:
					log.RInfo('{}: {} contents has been found'.format(sub, len(file.contents)))
				for mode in ['html', 'json', 'list', 'raw']:
					file.mode = mode
					file.write()
	log.RInfo('Start Old and Dead')
	for anal in analyzers:
		for since in [1, 365]:
			url_tests = anal.TEST
			for sub in url_tests:
				url = {anal.SITE: [sub]}
				old(url, since)
				dead(url)
	log.RInfo('Start Add Sub')
	for anal in analyzers:
		url = {anal.SITE: anal.TEST}
		add_sub(url, '')



import re
import json

from datetime     import datetime, timedelta
from ..core.tools import log
from .analyzer    import Analyzer
from ..downloader.youtube import (
	download_xml,
	download_html,
)
from ..core.sock  import (
	download_https
)
from ..core.download  import Download




class Youtube_Analyzer(Analyzer):
	SITE='[youtube]'
	URL_MATCH=r'(?:https://|)(?:www\.|m.|)youtube\.com/(?P<type>c/|channel/|user/|playlist\?list=|@)(?P<ID>[a-zA-Z0-9_-]*)'
	RE_CHANNEL=r'UC[A-Za-z0-9_-]{22}$'
	RE_PLAYLIST=r'PL[A-Za-z0-9_-]{32}$'
	TEST=[
		'https://www.youtube.com/channel/UCyg3MF1KU3dUK0HJBBoRYOw',
		'https://www.youtube.com/playlist?list=PL0H7ONNEUnnt59niYAZ07dFTi99u2L2n_',
		'https://www.youtube.com/channel/UCDlLfadiQHJuFkJnSBBnQsQ',
		'https://m.youtube.com/channel/UCDlLfadiQHJuFkJnSBBnQsQ',
		'https://www.youtube.com/c/RapidsCrew',
		'https://www.youtube.com/c/RestezChezVous',
		'UCyg3MF1KU3dUK0HJBBoRYOw',
		'PL0H7ONNEUnnt59niYAZ07dFTi99u2L2n_',
		'UCDlLfadiQHJuFkJnSBBnQsQ',
	]

	def __init__(self, url_id=''):
		self.id = self.extract_id(url_id)
		self.type = self._type_id(self.id)  # True --> chanel False -- > Playlist
		self.content = None

	def extract_id(self, url):
		if url == '':
			return ''
		match = self.match(url)
		if not match:
			return url
		if not re.match(self.RE_CHANNEL, match.group('ID')) and not re.match(self.RE_PLAYLIST, match.group('ID')):
			site = Download(True, 'www.youtube.com')
			path = re.findall('youtube.com(/.*)$', url)[0]
			site.download(path, headers={'Cookie': 'SOCS=CAESNQgDEitib3FfaWRlbnRpdHlmcm9udGVuZHVpc2VydmVyXzIwMjMwOTI2LjA1X3AwGgJmciACGgYIgITTqAY', 'Referer': 'https://consent.youtube.com/'})
			if site.status == '303' and 'Location' in site.headers:
				nurl = re.findall(r'Location: (.*)', site.headers)[0]
				path = re.findall('youtube.com(/.*)$', nurl)[0]
				site.download(path, headers={'Cookie': 'SOCS=CAESNQgDEitib3FfaWRlbnRpdHlmcm9udGVuZHVpc2VydmVyXzIwMjMwOTI2LjA1X3AwGgJmciACGgYIgITTqAY', 'Referer': 'https://consent.youtube.com/'})
			ids = re.findall(r'(UC[A-Za-z0-9_-]{22})', site.body)
			dup_uids = {}
			for uid in ids:
				if uid not in dup_uids:
					dup_uids[uid] = 1
				else:
					dup_uids[uid] += 1
			return max(dup_uids, key=dup_uids.get, default='')
		return match.group('ID')

	def add_sub(self, url):
		sub = self.extract_id(url)
		tid = self._type_id(sub)
		data = download_xml(sub, type_id=tid, split=False)
		if data is None:
			log.Error("The channel/playlist can't be add. It could be delete.")
			return None
		match = re.findall(r'<(?:title|name)>(.+?)</(?:title|name)>', data)
		if match != []:
			return sub + '\t' + match[0]
		log.Warning("The channel/playlist can't be add. It could be delete.")
		return None

	def real_analyzer(self):
		"""Recover all the videos of a channel or a playlist
		and add the informations in $HOME/.cache/youtube_sm/data/."""
		if self.method == '0':
			linfo = download_xml(self.id, self.type)
		elif self.method == '1':
			linfo = download_html(self.id, self.type)
		if linfo is None:
			return
		info = self.info_rss
		if self.method == '1':
			info = self.info_html
			if self.type:  # Channel
				linfo = linfo["contents"]["twoColumnBrowseResultsRenderer"]["tabs"][1]["tabRenderer"]["content"]["sectionListRenderer"]["contents"][0]["itemSectionRenderer"]["contents"][0]["gridRenderer"]["items"]
		for i in linfo:
			self.content = None
			info(i)
			if self.content is not None:
				self.write()

	def _full_url(self):
		if self.type:
			return 'https://www.youtube.com/channel/' + self.id
		return 'https://www.youtube.com/playlist?list=' + self.id

	def info_html(self, i):
		"""Recover the informations of the html page"""
		if self.type:  # Channel
			if "gridVideoRenderer" not in i:
				return 
			i = i["gridVideoRenderer"]
			self.content = {
				'url': i["videoId"],
				'title': i["title"]["runs"][0]["text"],
				'date': self.date_convert(i["publishedTimeText"]["simpleText"].replace('Streamed', '')),
				'uploader': re.findall('.* by (.+?) \d* \w* ago.*', i["title"]["accessibility"]["accessibilityData"]["label"])[0],
				# 'view': i["viewCountText"]["simpleText"],
			}
		else:  # Playlist
			self.content = self.info(i, {
				'url': {'re':r'data-video-id="(.{11})"'},
				'title': {'re':r'data-video-title="(.+?)"'},
				'uploader': {'re':r'data-video-username="(.+?)"'},
				'date': {'default': datetime.now()},
			})

	def info_rss(self, i):
		"""Recover the informations of a rss page"""
		self.content = self.info(i, {
			'url':         {'re': r'<yt:videoId>(.{11})</yt:videoId>'},
			'title':       {'re': r'<media:title>(.*)</media:title>'},
			'uploader':    {'re': r'<name>(.*)</name>'},
			'date':        {'re': r'<published>(.*)\+', 'date': '%Y-%m-%dT%H:%M:%S'},
			'view':        {'re': r'views="(.+?)"'},
		})
		if self.content is not None:
			self.content['view'] = self.content['view'].replace(',', '')

	def write(self):
		"""Write the information in a file"""
		self.content['image'] = 'https://i.ytimg.com/vi/{}/mqdefault.jpg'.format(self.content['url'])
		self.content['url'] = 'https://www.youtube.com/watch?v=' + self.content['url']
		self.content['url_uploader'] = self._full_url()
		self.file.add(**self.content)

	def _type_id(self, id):
		"""True = Channel; False = Playlist"""
		if re.match(self.RE_CHANNEL, id):
			return True
		elif re.match(self.RE_PLAYLIST, id):
			return False
		else:
			return True

	def date_convert(self, date):
		"""
		Convert "2 day ago" -> datetime format
		"""
		value, unit, _ = date.split()
		if unit in ['month', 'months']:
			unit = 'days'
			value = int(value) * 31
		elif unit in ['year', 'years']:
			unit = 'days'
			value = int(value) * 365
		elif unit in ['minute', 'hour', 'day', 'week']:
			unit += 's'
		delta = timedelta(**{unit: int(value)})
		date = datetime.now() - delta
		return date

	def old(self, sub, since):
		sub = self.extract_id(sub)
		data = download_xml(sub, self._type_id(sub))
		if data is None:
			self.subis(sub, self.ISDEAD)
		elif data == []:
			self.subis(sub, self.ISEMPTY)
		else:
			content = self.info(data[0], {
				'date': {'re': '<published>(.*)\+', 'date':'%Y-%m-%dT%H:%M:%S'},
				'name': {'re': '<name>(.+?)</name>'},
			})
			if content is None:
				log.Error('Fail to parse the rss feed')
			else:
				if since > content['date']:
					self.subis(content['name'], content['date'])
				else:
					self.subis(content['name'], self.ISOK)

	def dead(self, sub):
		""" Print the dead channel """
		sub = self.extract_id(sub)
		linfo = download_xml(sub, self._type_id(sub))
		if linfo is None:
			self.subis(sub, self.ISDEAD)
		elif linfo is []:
			self.subis(sub, self.ISEMPTY)
		else:
			self.subis(sub, self.ISOK)



from ..core.download import Download
from ..core.sock import *

import re
import json

def download_twitter(name):
	twitter = download_https('twitter.com', '/' + name)
	if twitter.status != '200':
		return
	token = re.findall('gt=([0-9]*)', twitter.body)[0]

	twimg = download_https('abs.twimg.com', '/responsive-web/web/main.22f2dd14.js')
	if twimg.status != '200':
		return None
	bearer = re.findall('s="Web-12",c="(.+?)"', twimg.body)[0]
	csrf = '4232262aca9e44937ce433580f1d8449'
	# token = '1263211448785866753'

	api = Download(True, 'api.twitter.com')

	api.download('/graphql/-xfUfZsnR_zqjFd-IfrN5A/UserByScreenName?variables=%7B%22screen_name%22%3A%22{}%22%2C%22withHighlightedLabel%22%3Atrue%7D'.format(name.lower()),
		headers={
			'content-type': 'application/json',
			'authorization': 'Bearer ' + bearer,
			'x-guest-token': token,
			'x-twitter-client-language': 'fr',
			'x-twitter-active-user': 'yes',
			'x-csrf-token': csrf,
			'origin': 'https://twitter.com',
			'referer': 'https://twitter.com/',
		},
		status='200'
	)
	if api.status != '200':
		return None
	uid = re.findall('"rest_id":"(.+?)"', api.body)[0]
	api.download('/2/timeline/profile/{}.json?include_profile_interstitial_type=1&include_blocking=1&include_blocked_by=1&include_followed_by=1&include_want_retweets=1&include_mute_edge=1&include_can_dm=1&include_can_media_tag=1&skip_status=1&cards_platform=Web-12&include_cards=1&include_composer_source=true&include_ext_alt_text=true&include_reply_count=1&tweet_mode=extended&include_entities=true&include_user_entities=true&include_ext_media_color=true&include_ext_media_availability=true&send_error_codes=true&simple_quoted_tweet=true&include_tweet_replies=false&userId={}&count=20&ext=mediaStats%2ChighlightedLabel%2CcameraMoment'.format(uid, uid),
		headers={
			'authorization': 'Bearer ' + bearer,
			'x-guest-token': token,
			'x-twitter-client-language': 'fr',
			'x-twitter-active-user': 'yes',
			'x-csrf-token': csrf,
			'origin': 'https://twitter.com',
			'referer': 'https://twitter.com/',
		},
		status='200'
	)
	if api.status == '403':
		data = json.loads(api.body)
		log.Error(name, ': ', data['errors'])
	if api.status != '200':
		return None
	return json.loads(api.body)



import re
import json

from ..core.tools import log
from ..core.sock  import (
	download_http,
	download_https
)
from ..core.download  import Download

def download_xml(url_id, type_id=True, split=True):
	"""
	Return a list of informations of each video with the
	RSS youtube page
	"""
	if type_id:  # Channel
		url = '/feeds/videos.xml?channel_id=' + url_id
	else:  # Playlist
		url = '/feeds/videos.xml?playlist_id=' + url_id
	site = download_http('www.youtube.com', url)
	if site.status != '200':
		return None
	data = site.body
	if split:
		linfo = data.split("<entry>")
		del linfo[0]
		return linfo
	else:
		return data


def download_html(url_id, type_id=True, split=True):
	"""
	Return a list of informations of each video with the
	current html page
	"""
	if type_id:
		url = '/channel/' + url_id + '/videos'
	else:
		site = download_https('www.youtube.com', '/playlist?list=' + url_id, useragent='youtube-sm')
		if site.status != '200':
			return None
		url = re.findall('<a href="(/watch\?v.+?)"', site.body)
		if url == []:
			log.Error('Fail to parse html page')
			return None
		url = url[0]
	site = Download(True, 'www.youtube.com')
	site.download(url, headers={'Cookie': 'CONSENT=YES+cb.20210413-13-p0.fr+FX+878', 'Referer': 'https://consent.youtube.com/'})
	if site.status != '200':
		return None
	data = site.body
	if split:
		if type_id:  # Channel
			lol = re.findall(">var ytInitialData = (\{.*});</script>", data)[0]
			#open('lol', 'w').write(lol)
			jso = json.loads(lol)
			return jso
	else:
		return data



import socket
import ssl
import re
import zlib

from .download import Download
from .tools import log

TIMEOUT = 2

def download_http(host, url, useragent=Download.USERAGENT):
	site = Download(False, host)
	site.USERAGENT = useragent
	site.download(url)
	return site

def download_https(host, url, useragent=Download.USERAGENT):
	site = Download(True, host)
	site.USERAGENT = useragent
	site.download(url)
	return site



from .analyzer	import Analyzer
from ..downloader.twitter import download_twitter
from ..core.tools import log
from datetime import datetime

import re

class Twitter_Analyzer(Analyzer):
	SITE='[twitter]'
	URL_MATCH=r'(?:https://|)(?:www\.|)twitter\.com/(?P<ID>[A-Za-z0-9_-]*)'
	TEST=[
		'https://twitter.com/UsulduFutur',
		'https://twitter.com/notweet4ever',
		'https://twitter.com/NoTweet',
		'UsulduFutur',
		'notweet4ever',
		'NoTweet',
	]

	def __init__(self, sub=''):
		self.id = self.extract_id(sub)

	def add_sub(self, sub):
		uid = self.extract_id(sub)
		if download_twitter(uid) is None:
			return None
		return uid + '\t '

	def real_analyzer(self):
		data = download_twitter(self.id)
		if data is None:
			return
		for tweet in data['globalObjects']['tweets'].values():
			content = {
				'url': 'https://twitter.com/{}/status/{}'.format(self.id, tweet['id_str']),
				'url_uploader': 'https://twitter.com/{}'.format(self.id),
				'title': tweet['full_text'],
				'date': datetime.strptime(tweet['created_at'], '%a %b %d %H:%M:%S +0000 %Y'),
			 	'uploader': self.id,
				'image': self.NO_IMG,
			}
			if 'card' in tweet and 'binding_values' in tweet['card']:
				for var in tweet['card']['binding_values']:
					if 'image' in var and 'original' in var:
						content['image'] = tweet['card']['binding_values'][var]['image_value']['url']
						break
			self.file.add(**content)

	def old(self, url, since):
		sub = self.extract_id(url)
		data = download_twitter(sub)
		if data is None:
			self.subis(sub, self.ISDEAD)
		elif data['globalObjects']['tweets'] == {}:
			self.subis(sub, self.ISEMPTY)
		else:
			first = list(data['globalObjects']['tweets'].values())[0]
			date = datetime.strptime(first['created_at'], '%a %b %d %H:%M:%S +0000 %Y')
			if since > date:
				self.subis(sub, date)
			else:
				self.subis(sub, self.ISOK)

	def dead(self, url):
		sub = self.extract_id(url)
		data = download_twitter(sub)
		if data is None:
			self.subis(sub, self.ISDEAD)
		elif data['globalObjects']['tweets'] == {}:
			self.subis(sub, self.ISEMPTY)
		else:
			self.subis(sub, self.ISOK)



import re

from datetime import datetime
from .analyzer import Analyzer
from ..downloader.revolutionpermanente import download_xml_revolutionpermanente


class RevolutionPermanente_Analyzer(Analyzer):
	SITE='[revolutionpermanente]'
	URL_MATCH=r'(?:https://|)(?:www\.|)revolutionpermanente\.fr.*'
	TEST=[
		'https://revolutionpermanente.fr/'
		'www.revolutionpermanente.fr/'
	]

	def __init__(self, url_id=''):
		self.id = url_id

	def add_sub(self, url):
		""" This function return the informations wich are write in sub.swy ."""
		return 'Revolution Permanente'

	def real_analyzer(self):
		""" The main function wich retrieve the informations and and write it
		in a file"""
		data = download_xml_revolutionpermanente()
		if data is None:
			return
		for ele in data:
			self.content = self.info(ele, {
				'url':{'re':'<link>(.*)</link>'},
				'title':{'re':'<title>(.*)</title>'},
				'url_uploader':{'default':'https://www.revolutionpermanente.fr/'},
				'uploader':{'default':'Revolution Permanente'},
				'image':{'re':"img class='spip_logo spip_logo_right spip_logos' .* src='(.+?)'"},
				'date':{'re':'<dc:date>(.*)</dc:date>','date':'%Y-%m-%dT%H:%M:%SZ'},
			})
			if self.content is not None:
				self.file.add(**self.content)



from ..downloader.twitch import download_twitch
from .analyzer	import Analyzer
from datetime import datetime

import re

class Twitch_Analyzer(Analyzer):
	SITE='[twitch]'
	URL_MATCH=r'(?:https://|)(?:www\.|)twitch\.tv/(?P<ID>[a-zA-Z0-9]*)(?:/.*|)'
	TEST=[
		'https://www.twitch.tv/danycaligula',
		'https://www.twitch.tv/raz404',
	]

	def __init__(self, sub=''):
		self.id = self.extract_id(sub)

	def add_sub(self, sub):
		"""
		This function return the informations wich are write in sub.swy.
		"""
		id = self.extract_id(sub)
		if id is None:
			return None
		return id + '\t' + id

	def real_analyzer(self):
		"""
		The main function wich retrieve the informations and and write it
		in a file
		"""
		data = download_twitch(self.id)
		if data is None:
			return
		data = data[1]
		for element in data['itemListElement']:
			content = {
				'url':          element['url'],
				'title':        element['name'],
				'uploader':     self.id,
				'url_uploader': 'https://www.twitch.tv/' + self.id,
				'image':        element['thumbnailUrl'][1],
				'view':         element['interactionStatistic']['userInteractionCount'],
				'date':         datetime.strptime(element['uploadDate'], '%Y-%m-%dT%H:%M:%SZ'),
			}
			if content is not None and re.match('https://www.twitch.tv/.*', content['url']):
				self.file.add(**content)



import re
from datetime  import datetime
from threading import Thread
from time      import time
from ..core.tools import log


class Analyzer(Thread):
	"""
	Common Function to all Analyzers
	"""
	NO_IMG='https://sawyerf.gitlab.io/youtube_sm/pixel.png'
	def __sub_init__(self):
		self.method = '0'
		self.prog = None
		self.file = None

	def initialize(self, method='0', file=None, prog=None):
		self.method = method
		self.prog = prog
		self.file = file

	def Thread(self):
		Thread.__init__(self)

	def run(self):
		start = time()
		self.real_analyzer()
		if self.prog is not None:
			self.prog.add()
		diff = time() - start
		if diff > 5:
			if hasattr(self, 'id'):
				log.RWarning(self.SITE, ' `', self.id, '`: This sub took {:.2} seconds to complete'.format(diff))
			else:
				log.RWarning(self.SITE, ' This sub took {:.2} seconds to complete'.format(diff))

	def match(self, url):
		return re.match(self.URL_MATCH, url)

	def extract_id(self, url):
		if url == '':
			return ''
		match = self.match(url)
		if not match:
			return url
		return match.group('ID')

	def info(self, item, matchs):
		content = {}
		for var, match in matchs.items():
			if 're' not in match:
				if 'default' in match:
					content[var] = match['default']
					continue
				else:
					return None
			find = re.findall(match['re'], item)
			i = 0
			if 'i' in match:
				i = match['i']
			if len(find) <= i:
				if 'default' not in match:
					return None
				content[var] = match['default']
			else:
				content[var] = find[i]
				if 'func' in match:
					content[var] = match['func'](content[var])
					if content[var] is None:
						return None
				elif 'date' in match:
					try:
						content[var] = datetime.strptime(content[var], match['date'])
					except:
						return None
		return content

	ISOK=0
	ISEMPTY=1
	ISDEAD=2
	def subis(self, text, status):
		if status == self.ISDEAD:
			log.Error('[CHANNEL DEAD] ', text)
		elif status == self.ISEMPTY:
			log.Error('[ NO CONTENT ] ', text)
		elif type(status) == datetime:
			log.RWarning('[ {} ] {}'.format(status.strftime('%Y-%m-%d'), text))
		elif status == self.ISOK:
			log.RInfo('[     OK     ] ', text)

	def old(self, sub, since):
		pass

	def dead(self, sub):
		pass



from ..core.sock import download_https
from ..core.tools import log


def download_xml_peertube(url_id):
	info = url_id.split(':')
	path = '/feeds/videos.xml'
	if len(info) == 2 and info[1] != '':
		path += '?accountId=' + info[1]

	site = download_https(info[0], path)
	if site.status != '200':
		site = download_https(info[0], path.replace('accountId', 'videoChannelId'))
		if site.status != '200':
			log.Error('Failed to download ({})'.format(info))
			return None
	data = site.body.split('<item>')
	del data[0]
	return data



import re
import locale
from datetime  import datetime

from ..core.tools import log
from ..downloader.evous import download_html_evous
from .analyzer	import Analyzer

class Evous_Analyzer(Analyzer):
	SITE='[evous]'
	URL_MATCH=r'(?:https://|)(?:www\.|)evous\.fr(?:/.*|)'
	TEST=[
		'https://www.evous.fr/',
		'https://www.evous.fr/Les-Manifestations-a-Paris-la-semaine-1176044.html',
	]

	def __init__(self, url_id=''):
		pass

	def add_sub(self, sub):
		"""
		This function return the informations wich are write in sub.swy.
		"""
		return 'evous'

	def real_analyzer(self):
		"""
		The main function wich retrieve the informations and and write it
		in a file
		"""
		infos = download_html_evous()
		loc = locale.getlocale()
		try:
			locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')
		except locale.Error as e:
			log.Error("Evous: ", e, ": You need to add `fr_FR.UTF-8`")
			return
		for day in infos:
			manifs = re.findall(r'(?:<br\ \/>.+?\n)', day.group('manifs'))
			dateManif = datetime.strptime(day.group('date'), '%A %d %B %Y')
			for manif in manifs:
				self.content = self.info(manif, {
					'url': {'re':'<a href="(.+?)" class', 'default': ''},
					'title': {'re':'<strong>(.+?)</strong>'},
					'url_uploader': {'default':'https://www.evous.fr/Les-Manifestations-a-Paris-la-semaine-1176044.html'},
					'uploader': {'default':'Evous'},
					'image': {'default':"http://img.over-blog-kiwi.com/1/49/25/24/20160318/ob_bac2c2_code19.jpg"},
					# 'date': {'re':'</strong> le (\w* \d{1,2} \w* \d{4})', 'date':'%A %d %B %Y', 'default': dateManif},
					'date': {'default': dateManif},
				})
				if self.content is not None:
					self.content['title'] = re.sub(r'<.+?>', '', self.content['title'])
					# if re.match(r'<a .*>.+?</a>', self.content['title']):
					# 	self.content['title'] = re.findall(r'<a .*>(.+?)</a>', self.content['title'])[0]
					if re.match(r'^/.*', self.content['url']):
						self.content['url'] = 'https://www.evous.fr' + self.content['url']
					self.file.add(**self.content)
		locale.setlocale(locale.LC_ALL, loc)



import re
import json

from ..core.sock import download_https


def download_twitch(sid):
	site = download_https('www.twitch.tv', '/{}/videos?filter=all'.format(sid))
	if site.status != '200':
		return None
	data = re.findall('<script type="application/ld\+json">(.+?)</script>', site.body)
	if data == []:
		return None
	return json.loads(data[0])



import re
from ..core.sock import download_https


def download_html_evous(split=True):
	site = download_https("www.evous.fr", "/Les-Manifestations-a-Paris-la-semaine-1176044.html")
	if site.status != '200':
		return None
	data = site.body
	if split:
			linfo = re.finditer('<strong>(?P<date>[A-Za-z]+? [0-9]+? [a-z]+? [0-9]{4})<\/strong>\n(?P<manifs>.+?)<p class="land-see-hero-caption', data, re.DOTALL)
			if linfo == []:
					return None
			return linfo
	return data
